@@ -11,8 +11,8 @@
 ** Wyatt Banks [wbanks@crucialsecurity.com]
 ** Copyright (c) 2005 Crucial Security Inc.  All rights reserved.
 **
-** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
+** Brian Carrier [carrier <at> sleuthkit [dot] org]
+** Copyright (c) 2003-2011 Brian Carrier.  All rights reserved
 **
 ** Copyright (c) 1997,1998,1999, International Business Machines
 ** Corporation and others. All Rights Reserved.
@@ -57,12 +57,11 @@
 **        each copy of the program.
 */
 
-/** 
- * \file tsk_iso9660.h
+/* 
  * Contains the structures and function APIs for ISO9660 file system support.
  */
 
-/* refernece documents used:
+/* reference documents used:
  * IEEE P1281 - System Use Sharing Protocol, version 1.12
  * IEEE P1282 - Rock Ridge Interchange Protocol, version 1.12
  * ECMA-119 - Volume and File Structure of CDROM for Information Interchange,
@@ -80,8 +79,7 @@
  */
 #define ISO9660_FIRSTINO	0
 #define ISO9660_ROOTINO		0
-#define ISO9660_NIADDR		0       /* iso9660 doesnt have indirect blocks */
-#define ISO9660_NDADDR		1       /* just a single data "block" */
+#define ISO9660_FILE_CONTENT_LEN sizeof(TSK_DADDR_T)
 #define ISO9660_SBOFF		32768
 #define ISO9660_SSIZE_B		2048
 #define ISO9660_MIN_BLOCK_SIZE	512
@@ -99,6 +97,7 @@
 
 #define ISO9660_MAXNAMLEN_STD	128     ///< Maximum length of standard name
 #define ISO9660_MAXNAMLEN_JOL   128     ///< maximum UTF-16 joliet name (in bytes)
+#define ISO9660_MAXNAMLEN_RR    256     ///< Maximum length of a Rock Ridge extension name
 #define ISO9660_MAXNAMLEN   (ISO9660_MAXNAMLEN_JOL << 1)        // mult jol by 2 to account for UTF-16 to UTF-8 conversion
 
 /* Bits in permissions used in extended attribute records.  */
@@ -134,6 +133,10 @@
 #define ISO9660_CTYPE_ASCII 0
 #define ISO9660_CTYPE_UTF16 1
 
+
+
+
+
 /* recording date and time */
 typedef struct {
     uint8_t year;               /* years since 1900 */
@@ -142,23 +145,9 @@ typedef struct {
     uint8_t hour;               /* 0-23 */
     uint8_t min;                /* 0-59 */
     uint8_t sec;                /* 0-59 */
-    uint8_t gmt_off;            /* greenwich mean time offset */
+    int8_t gmt_off;             /* greenwich mean time offset (in 15 minute intervals) */
 } record_data;
 
-/* data and time format
- * all are stored as "digits" according to specifications for iso9660
- */
-typedef struct {
-    uint8_t year[4];            /* 1 to 9999 */
-    uint8_t month[2];           /* 1 to 12 */
-    uint8_t day[2];             /* 1 to 31 */
-    uint8_t hour[2];            /* 0 to 23 */
-    uint8_t min[2];             /* 0 to 59 */
-    uint8_t sec[2];             /* 0 to 59 */
-    uint8_t hun[2];             /* hundredths of a second */
-    uint8_t gmt_off;            /* GMT offset */
-} date_time;
-
 /* iso 9660 directory record */
 typedef struct {
     uint8_t entry_len;          /* length of directory record */
@@ -167,7 +156,7 @@ typedef struct {
     uint8_t ext_loc_m[4];       /* location of extent - be */
     uint8_t data_len_l[4];      /* data length - le */
     uint8_t data_len_m[4];      /* data length - be */
-    record_data rec_time;       /* recording date and time */
+    record_data rec_time;       /* recording date and time (7 bytes) */
     int8_t flags;               /* file flags */
     uint8_t unit_sz;            /* file unit size */
     uint8_t gap_sz;             /* interleave gap size */
@@ -195,6 +184,22 @@ typedef struct {
     char name;
 } iso9660_root_dentry;
 
+
+/* data and time format
+ * all are stored as "digits" according to specifications for iso9660
+ */
+typedef struct {
+    uint8_t year[4];            /* 1 to 9999 */
+    uint8_t month[2];           /* 1 to 12 */
+    uint8_t day[2];             /* 1 to 31 */
+    uint8_t hour[2];            /* 0 to 23 */
+    uint8_t min[2];             /* 0 to 59 */
+    uint8_t sec[2];             /* 0 to 59 */
+    uint8_t hun[2];             /* hundredths of a second */
+    uint8_t gmt_off;            /* GMT offset */
+} date_time;
+
+
 /* generic volume descriptor */
 typedef struct {
     uint8_t type;               ///<  volume descriptor type 
@@ -293,7 +298,7 @@ typedef struct {
     uint8_t len_di;             /* length of directory identifier */
     uint8_t attr_len;           /* extended attribute record length */
     uint8_t ext_loc[4];         /* location of extent */
-    uint8_t par_dir[2];         /* parent directory number */
+    uint8_t par_dir[2];         /* parent directory number (its entry in the path table) */
 } path_table_rec;
 
 /* extended attribute record */
@@ -314,6 +319,21 @@ typedef struct {
     uint8_t len_esc[ISODCL(182, 182)];  /* length of escape sequences */
 } ext_attr_rec;
 
+
+#define ISO_EA_IRSYS    0x0001
+#define ISO_EA_IWSYS    0x0002
+#define ISO_EA_IXSYS    0x0004
+#define ISO_EA_IRUSR    0x0010
+#define ISO_EA_IWUSR    0x0020
+#define ISO_EA_IXUSR    0x0040
+#define ISO_EA_IRGRP    0x0100
+#define ISO_EA_IWGRP    0x0200
+#define ISO_EA_IXGRP    0x0400
+#define ISO_EA_IROTH    0x1000
+#define ISO_EA_IWOTH    0x2000
+#define ISO_EA_IXOTH    0x4000
+
+
 /* primary volume descriptor linked list node */
 typedef struct iso9660_pvd_node {
     iso9660_pvd pvd;
@@ -329,15 +349,14 @@ typedef struct iso9660_svd_node {
 /* RockRidge extension info */
 typedef struct {
     TSK_UID_T uid /* owner */ ;
-    TSK_GID_T gid;                  /* group */
+    TSK_GID_T gid;              /* group */
     uint16_t mode;              /* posix file mode */
     uint32_t nlink;             /* number of links */
-    char fn[ISO9660_MAXNAMLEN_STD];     /* alternate filename */
+    char fn[ISO9660_MAXNAMLEN_RR];     /* alternate filename */
 } rockridge_ext;
 
 /** \internal
  * Internally used structure to hold basic inode information.
- *
  */
 typedef struct {
     iso9660_dentry dr;          /* directory record */
@@ -345,16 +364,18 @@ typedef struct {
     char fn[ISO9660_MAXNAMLEN + 1];     /* file name */
     rockridge_ext *rr;          /* RockRidge Extensions */
     int version;
-    TSK_OFF_T susp_off;             ///< Byte offset in image of SUSP (or NULL)
-    TSK_OFF_T susp_len;             ///< Length in bytes of SUSP
+    uint8_t is_orphan;          /* 1 if the file was found from processing other volume descriptors besides the first one, 0 otherwise */
+    TSK_OFF_T susp_off;         ///< Byte offset in image of SUSP (or NULL)
+    TSK_OFF_T susp_len;         ///< Length in bytes of SUSP
 } iso9660_inode;
 
 /* inode linked list node */
 typedef struct iso9660_inode_node {
     iso9660_inode inode;
-    TSK_OFF_T offset;               /* byte offset of inode into disk */
-    TSK_INUM_T inum;                /* identifier of inode */
-    int size;                   /* kludge: used to flag fifos, etc */
+    TSK_OFF_T offset;           /* byte offset of first block of file in file system */
+    TSK_OFF_T dentry_offset;    /* byte offset of directory entry structure in file system */
+    TSK_INUM_T inum;            /* identifier of inode (assigned by TSK) */
+    int size;                   /* number of bytes in file */
     int ea_size;                /* length of ext attributes */
     struct iso9660_inode_node *next;
 } iso9660_inode_node;
@@ -362,8 +383,6 @@ typedef struct iso9660_inode_node {
 /* The all important ISO_INFO struct */
 typedef struct {
     TSK_FS_INFO fs_info;        /* SUPER CLASS */
-    TSK_INUM_T dinum;               /* cached inode number */
-    iso9660_inode *dinode;      /* cached disk inode */
     uint32_t path_tab_addr;     /* address of path table */
     uint32_t root_addr;         /* address of root dir extent */
     iso9660_pvd_node *pvd;      ///< Head of primary volume descriptor list (there should be only one...)
@@ -372,11 +391,13 @@ typedef struct {
     uint8_t rr_found;           /* 1 if rockridge found */
 } ISO_INFO;
 
-extern uint8_t iso9660_dent_walk(TSK_FS_INFO * fs, TSK_INUM_T inode,
-    TSK_FS_DENT_FLAG_ENUM flags, TSK_FS_DENT_TYPE_WALK_CB action,
-    void *ptr);
+extern TSK_RETVAL_ENUM iso9660_dir_open_meta(TSK_FS_INFO * a_fs,
+    TSK_FS_DIR ** a_fs_dir, TSK_INUM_T a_addr, int recursion_depth);
 
-extern uint8_t iso9660_dinode_load(ISO_INFO * iso, TSK_INUM_T inum);
+extern uint8_t iso9660_dinode_load(ISO_INFO * iso, TSK_INUM_T inum,
+    iso9660_inode * dinode);
+
+extern int iso9660_name_cmp(TSK_FS_INFO *, const char *, const char *);
 
 /**********************************************************
  *
